@@ -1,18 +1,57 @@
 import socketio
 from typing import Any
 import logging
-sio = socketio.Client()
+from matplotlib import pyplot as plt
+import io
+import sys
+import base64
 
+sio = socketio.Client()
 task_id = None
 # target log for where intercepted logs should be forwarded to
 logging_target = None
 
+timeout_count = 0
+
+def encode_image(arr,
+title: str = '',
+format : str = 'jpg',
+figsize: tuple = (8, 8),
+cmap: str = "gray"):
+    fig = plt.figure(figsize=figsize)
+    plt.imshow(arr, cmap=cmap)
+    plt.suptitle(title, fontsize=16)
+    my_stringIObytes = io.BytesIO()
+    plt.savefig(my_stringIObytes, format=format)
+    my_stringIObytes.seek(0)
+    my_base64_jpgData = base64.b64encode(my_stringIObytes.read()).decode()
+    return my_base64_jpgData
+
+# https://stackoverflow.com/a/51981833
+def vsui_process():
+    def decorator(func):
+        def wrapper(*args, **kwargs):
+            r = None
+            try:
+                r = func(*args, **kwargs)
+            except Exception as e:
+                r = disconnect()
+            finally:
+                return r
+        return wrapper
+    return decorator
+
 def safe_emit(event, data):
     ''' emit a socketio event with exception handling '''
+    global timeout_count
     try:
         sio.emit(event, data)
+        timeout_count = 0
     except:
-        pass
+        timeout_count += 1
+        if timeout_count == 5:
+            disconnect()
+            sys.exit(1)
 
 def connect(HOST : str = 'localhost', PORT : str = '8000') -> None:
     ''' initiate a socketio session with a server '''
